@@ -69,7 +69,7 @@ module Network.Gitit.ContentTransformer
   )
 where
 
-import Control.Exception (throwIO, catch)
+import qualified Control.Exception as E
 import Control.Monad.State
 import Control.Monad.Reader (ask)
 import Data.Foldable (traverse_)
@@ -86,7 +86,6 @@ import Network.Gitit.Types
 import Network.HTTP (urlDecode)
 import Network.URI (isUnescapedInURI)
 import Network.URL (encString)
-import Prelude hiding (catch)
 import System.FilePath
 import qualified Text.Pandoc.Builder as B
 import Text.HTML.SanitizeXSS (sanitizeBalance)
@@ -108,6 +107,12 @@ import qualified Data.FileStore as FS
 import qualified Text.Pandoc as Pandoc
 import Text.URI (parseURI, URI(..), uriQueryItems)
 
+#if MIN_VERSION_pandoc(1,14,0)
+import Text.Pandoc.Error (handleError)
+#else
+handleError :: Pandoc -> Pandoc
+handleError = id
+#endif
 --
 -- ContentTransformer runners
 --
@@ -274,8 +279,8 @@ rawContents = do
   file <- getFileName
   fs <- lift getFileStore
   let rev = pRevision params
-  liftIO $ catch (liftM Just $ FS.retrieve fs file rev)
-                 (\e -> if e == FS.NotFound then return Nothing else throwIO e)
+  liftIO $ E.catch (liftM Just $ FS.retrieve fs file rev)
+               (\e -> if e == FS.NotFound then return Nothing else E.throwIO e)
 
 --
 -- Response-generating combinators
@@ -670,15 +675,23 @@ readerFor pt lhs =
                  , readerExtensions = if lhs
                                          then Set.insert Ext_literate_haskell
                                               $ readerExtensions def
-                                         else readerExtensions def }
-  in case pt of
-       RST      -> readRST defPS
-       Markdown -> readMarkdown defPS
-       LaTeX    -> readLaTeX defPS
-       HTML     -> readHtml defPS
-       Textile  -> readTextile defPS
-       Org      -> readOrg defPS
-       DocBook  -> readDocBook defPS
+                                         else readerExtensions def
+                 , readerParseRaw = True
+                 }
+  in handleError . case pt of
+       RST        -> readRST defPS
+       Markdown   -> readMarkdown defPS
+#if MIN_VERSION_pandoc(1,14,0)
+       CommonMark -> readCommonMark defPS
+#else
+       CommonMark -> error "CommonMark input requires pandoc 1.14"
+#endif
+       LaTeX      -> readLaTeX defPS
+       HTML       -> readHtml defPS
+       Textile    -> readTextile defPS
+       Org        -> readOrg defPS
+       DocBook    -> readDocBook defPS
+       MediaWiki  -> readMediaWiki defPS
 
 wikiLinksTransform :: Pandoc -> PluginM Pandoc
 wikiLinksTransform pandoc
