@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 {-
 Copyright (C) 2009 John MacFarlane <jgm@berkeley.edu>
 
@@ -124,13 +125,18 @@ respondSlides templ slideVariant page doc = do
                         $ dropWhile (not . isPrefixOf "<!-- {{{{ dzslides core")
                         $ lines dztempl
                   else return ""
-    let h = writeHtmlString opts'{
+    let opts'' = opts'{
                 writerVariables =
                   ("body",body''):("dzslides-core",dzcore):("highlighting-css",pygmentsCss):variables'
                ,writerTemplate = template
                ,writerUserDataDir = pandocUserData cfg
-               } (Pandoc meta [])
+               }
+    let h = writeHtmlString opts'' (Pandoc meta [])
+#if MIN_VERSION_pandoc(1,13,0)
+    h' <- liftIO $ makeSelfContained opts'' h
+#else
     h' <- liftIO $ makeSelfContained (pandocUserData cfg) h
+#endif
     ok . setContentType "text/html;charset=UTF-8" .
       -- (setFilename (page ++ ".html")) .
       toResponseBS B.empty $ fromString h'
@@ -156,6 +162,12 @@ respondMarkdown :: String -> Pandoc -> Handler
 respondMarkdown = respondS "markdown" "text/plain; charset=utf-8" ""
   writeMarkdown defaultRespOptions{writerReferenceLinks = True}
 
+#if MIN_VERSION_pandoc(1,14,0)
+respondCommonMark :: String -> Pandoc -> Handler
+respondCommonMark = respondS "commonmark" "text/plain; charset=utf-8" ""
+  writeCommonMark defaultRespOptions{writerReferenceLinks = True}
+#endif
+
 respondPlain :: String -> Pandoc -> Handler
 respondPlain = respondS "plain" "text/plain; charset=utf-8" ""
   writePlain defaultRespOptions
@@ -279,6 +291,9 @@ exportFormats cfg = if pdfExport cfg
                 , ("Texinfo",   respondTexinfo)
                 , ("reST",      respondRST)
                 , ("Markdown",  respondMarkdown)
+#if MIN_VERSION_pandoc(1,14,0)
+                , ("CommonMark",respondCommonMark)
+#endif
                 , ("Plain text",respondPlain)
                 , ("MediaWiki", respondMediaWiki)
                 , ("Org-mode",  respondOrg)
