@@ -60,7 +60,7 @@ import Network.Gitit.State
 import Network.Gitit.Types
 import Data.FileStore
 import Data.Char (toLower)
-import Control.Monad (mzero, liftM, unless, MonadPlus)
+import Control.Monad (mzero, liftM, unless)
 import qualified Data.Map as M
 import qualified Data.ByteString.UTF8 as UTF8
 import qualified Data.ByteString.Lazy.UTF8 as LazyUTF8
@@ -106,7 +106,9 @@ withUserFromSession handler = withData $ \(sk :: Maybe SessionKey) -> do
             Nothing    -> return Nothing
             Just sd    -> do
               addCookie (MaxAge $ sessionTimeout cfg) (mkCookie "sid" (show $ fromJust sk))  -- refresh timeout
-              getUser $! sessionUser sd
+              case sessionUser sd of
+                Nothing -> return Nothing
+                Just user -> getUser user
   let user = maybe "" uUsername mbUser
   localRq (setHeader "REMOTE_USER" user) handler
 
