@@ -21,7 +21,46 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 {- | Types for Gitit modules.
 -}
 
-module Network.Gitit.Types where
+module Network.Gitit.Types (
+                            PageType(..)
+                           , FileStoreType(..)
+                           , MathMethod(..)
+                           , AuthenticationLevel(..)
+                           , Config(..)
+                           , Page(..)
+                           , SessionKey
+                           -- we do not export SessionData constructors, in case we need to extend  SessionData with other data in the future
+                           , SessionData
+                           , sessionData
+                           , sessionDataGithubState
+                           , sessionUser
+                           , sessionGithubState
+                           , User(..)
+                           , Sessions(..)
+                           , Password(..)
+                           , GititState(..)
+                           , HasContext
+                           , modifyContext
+                           , getContext
+                           , ContentTransformer
+                           , Plugin(..)
+                           , PluginData(..)
+                           , PluginM
+                           , runPluginM
+                           , Context(..)
+                           , PageLayout(..)
+                           , Tab(..)
+                           , Recaptcha(..)
+                           , Params(..)
+                           , Command(..)
+                           , WikiState(..)
+                           , GititServerPart
+                           , Handler
+                           , fromEntities
+                           , GithubConfig
+                           , oAuth2
+                           , org
+                           , githubConfig) where
 
 import Control.Monad.Reader (ReaderT, runReaderT, mplus)
 import Control.Monad.State (StateT, runStateT, get, modify)
@@ -30,15 +69,29 @@ import System.Log.Logger (Priority(..))
 import Text.Pandoc.Definition (Pandoc)
 import Text.XHtml (Html)
 import qualified Data.Map as M
+import Data.Text (Text)
 import Data.List (intersect)
 import Data.Time (parseTime)
+#if MIN_VERSION_time(1,5,0)
+import Data.Time (defaultTimeLocale)
+#else
 import System.Locale (defaultTimeLocale)
+#endif
 import Data.FileStore.Types
 import Network.Gitit.Server
 import Text.HTML.TagSoup.Entity (lookupEntity)
 import Data.Char (isSpace)
+import Network.OAuth.OAuth2
 
-data PageType = Markdown | RST | LaTeX | HTML | Textile | Org | DocBook
+data PageType = Markdown
+              | CommonMark
+              | RST
+              | LaTeX
+              | HTML
+              | Textile
+              | Org
+              | DocBook
+              | MediaWiki
                 deriving (Read, Show, Eq)
 
 data FileStoreType = Git | Darcs | Mercurial deriving Show
@@ -149,7 +202,10 @@ data Config = Config {
   -- | Filter HTML through xss-sanitize
   xssSanitize          :: Bool,
   -- | The default number of days in the past to look for \"recent\" activity
-  recentActivityDays   :: Int
+  recentActivityDays   :: Int,
+  -- | Github client data for authentication (id, secret, callback,
+  -- authorize endpoint, access token endpoint)
+  githubAuth           :: GithubConfig
   }
 
 -- | Data for rendering a wiki page.
@@ -167,9 +223,16 @@ data Page = Page {
 type SessionKey = Integer
 
 data SessionData = SessionData {
-  sessionUser :: String
+  sessionUser :: Maybe String,
+  sessionGithubState :: Maybe String
 } deriving (Read,Show,Eq)
 
+sessionData :: String -> SessionData
+sessionData user = SessionData (Just user) Nothing
+
+sessionDataGithubState  :: String -> SessionData
+sessionDataGithubState  githubState = SessionData Nothing (Just githubState)
+
 data Sessions a = Sessions {unsession::M.Map SessionKey a}
   deriving (Read,Show,Eq)
 
@@ -416,3 +479,9 @@ fromEntities ('&':xs) =
 fromEntities (x:xs) = x : fromEntities xs
 fromEntities [] = []
 
+data GithubConfig = GithubConfig { oAuth2 :: OAuth2
+                                 , org :: Maybe Text
+                                 }
+
+githubConfig :: OAuth2 -> Maybe Text -> GithubConfig
+githubConfig = GithubConfig
