@@ -1,3 +1,5 @@
+{-# LANGUAGE PackageImports #-}
+
 -- Usage:  a paragraph containing just [My page](!subst)
 -- will be replaced by the contents of My page.
 --
@@ -6,7 +8,7 @@
 
 module Subst (plugin) where
 
-import Control.Monad.CatchIO (try)
+import "MonadCatchIO-mtl" Control.Monad.CatchIO (try)
 import Data.FileStore (FileStoreError, retrieve)
 import Text.Pandoc (def, readMarkdown)
 import Network.Gitit.ContentTransformer (inlinesToString)
